#!/usr/bin/perl -w
#
# Creates photo album from group of JPEGS
# Kevin Beason 10/17/01
#
# Usage: make.pl [ -h ] [ -t title ] [ -l list_filename ]
#                [ -x cols ] [ -y rows ] [ -r xDim yDim ] file1 file2 ...
#
# Depends on ImageMagick's convert

use strict;

my $title = "";
my $images = ""; # images subdirectory, e.g. "images/"
my $convert = "convert";
my $imagesurl = "/~beason/images";
my ($i, $j, $k);
my $cols = 4;
my $rows = 4;
my @files;
my @htmls;
my @thumbs;
my @descs;
my @origs;
my %resize = ( doit => 0,
               x => 0,
               y => 0 );

# make sure convert is in path
@_ = split /:/, $ENV{"PATH"};
my $in_path = 0;
foreach (@_){
    $_ = "$_/$convert";
    if (-x $_){
	$in_path++;
	last;
    }
}
die "Cannot find convert command." unless ($in_path);

# show help
for ($i=0; $i<@ARGV;){
    if ($ARGV[$i] eq "-h"){
        $cols = $ARGV[$i+1];
	splice @ARGV, $i, 1;
	
        print <<USAGE;

makegallery.pl
Kevin Beason (10/17/01)
	  
Creates HTML photo album from group of JPEGS.
Requires "convert", by ImageMagick (http://www.imagemagick.org/).

Synopsis:
	  
  make.pl [-h] [ -t title ] [ -l list_filename ]
          [ -x cols ] [ -y rows ] [ -r xDim yDim ] file1 file2 ...
	  
Options:
	  
  -t title      Title to use for gallery.
  -l filename   File containing list of images to use.
  -x cols       Columns to use in thumbnail gallery.
  -y rows       Rows to use in thumbnail gallery.
  -r xDim yDim  Display photos using (xDim,yDim) resolution,
	        by using resizing.

USAGE

	exit;
    }
    else {
	$i++;
    }
}

# read cols
for ($i=0; $i<@ARGV;){
    if ($ARGV[$i] eq "-x"){
        $cols = $ARGV[$i+1];
	splice @ARGV, $i, 2;
    }
    else {
	$i++;
    }
}

# read rows
for ($i=0; $i<@ARGV;){
    if ($ARGV[$i] eq "-y"){
        $rows = $ARGV[$i+1];
	splice @ARGV, $i, 2;
    }
    else {
         $i++;
    }
}

# read resize
for ($i=0; $i<@ARGV;){
    if ($ARGV[$i] eq "-r"){
        $resize{x} = $ARGV[$i+1];
        $resize{y} = $ARGV[$i+2];
	$resize{doit}++;
	splice @ARGV, $i, 3;
    }
    else {
	$i++;
    }
}

# read title
for ($i=0; $i<@ARGV;){
    if ($ARGV[$i] eq "-t"){
        $title = $ARGV[$i+1];
	splice @ARGV, $i, 2;
    }
    else {
	$i++;
    }
}

# read file list
for ($i=0; $i<@ARGV;){
    if ($ARGV[$i] eq "-l"){
        $_ = $ARGV[$i+1];
	splice @ARGV, $i, 2;
	
	# read filenames from file list, one per line
	open(FILELIST, "<$_") or die "Could not open file $_: $!";
	while (<FILELIST>){
	    chomp;
	    push @files, $_;
	}
	close FILELIST;
    }
    else {
	# assume argument is filename, add to list
        push @files, $ARGV[$i];
	$i++;
    }
}

# weed out thumbnails
my @skipped;
my @cache = @files;
for ($i=0; $i<@files; $i++){
    for ($j=0; $j<@cache; $j++){
	$cache[$j] =~ /^(.*)\.(jpg|JPG|jpeg|JPEG)$/ or next;
	if ($files[$i] =~ /^$1_[t|r]\.$2$/){
	    # $files[$i] is a thumbnail
	    push @skipped, $files[$i];
	    splice @files, $i, 1;
	    $i--;
	    last;
	}
    }
}
if (@skipped){
    print "Skipped these images because they are thumbnails or resizes:\n",
        join(", ", @skipped), "\n";
}

# make sure there are files specified
do {
    print "Usage: make.pl [ -t title ] [ -l list_filename ]\n";
    print "               [ -x cols ] [ -y rows ] [ -r xDim yDim ] file1 file2 ...\n";
    exit;
} unless @files;

# initialize @htmls and @thumbs (image.html and image_t.jpg)
# and @descs (descriptions)
for ($i=0; $i<@files; $i++){
    $files[$i] =~ /^(.*)\.(jpg|JPG|jpeg|JPEG)$/ or
        die "Invalid filename: '$files[$i]'";
    
    # Resize image
    if ($resize{doit}){ 
	$origs[$i] = $files[$i];
	$files[$i] = "$1_r.$2";
	if (-e $files[$i]){
	    print "Skipping resizing image: $origs[$i]\n";
	}
	else {
	    print "Resizing image: $origs[$i]\n";
	    system($convert, "-geometry", $resize{x}."x".$resize{y}, $origs[$i], $files[$i]);
	}
    }
	
    $htmls[$i] = "$1.html";
    $thumbs[$i] = "$1_t.$2";
    $descs[$i] = "$1.desc";
}

# open links file
open(LINKS, ">links.html") or die $!;

my ($orig1, $orig2);
my ($up1, $up2);
my $just_one = (scalar(@files) == 1);

# create page for each image
for ($i=0; $i<@files; $i++){

    # helpful notice
    print "Making photo pg.@{[$i+1]}: ", $htmls[$i], ". ";
    
    # open the page for writing.
    open(PAGE, ">$htmls[$i]") or die $!;

    
    # create thumbnail
    print "Making Tnail: ";
    if (! -e $thumbs[$i] ||                   # if thumbnail doesn't exit
	(stat($files[$i]))[9] > (stat($thumbs[$i]))[9] ){ # or if file newer
	print "Yes";
	    
	# first, shrink to TIF
	$_ = $thumbs[$i] . ".TIF";
        system($convert, "-geometry", "100x100", $files[$i], $_);
	
	# second, compress to JPEG
        system($convert, $_, $thumbs[$i]);
	    
	# third, remove TIF
	unlink($_) or die "Unable to remove $_: $!";
    }
    else {
	print "No";
    }
    print ". ";
    
    # create description
    if (! -e $descs[$i]){
        system("touch", $descs[$i]);
    }
    
    # Link to original photo
    if ($origs[$i]){
	$orig1 = "&nbsp; &nbsp; &nbsp; <a href=\"$origs[$i]\">" .
	         "<font size=2><img src=\"$imagesurl/magnify.gif\"" .
	         "align=absmiddle border=0>" .
                 "<b>View Large</b></font></a>";
	$up1 = "&nbsp; &nbsp; &nbsp; " .
   	       "<a href=\"index" . (int($i/($rows*$cols))+1) . ".html" .
	       "\">" .
	       "<font size=2><img src=\"$imagesurl/up_arrow.gif\"" .
               "align=absmiddle border=0>" .
               "<b>Thumbnails</b></font></a>";
	$orig2 = "<br><a href=\"$origs[$i]\">" .
	         "<font size=2><img src=\"$imagesurl/magnify.gif\"" .
	         "align=absmiddle border=0>" .
                 "<b>View Large Original</b></font></a>";
	$up2 = "<br><a href=\"index" . (int($i/($rows*$cols))+1) . ".html" .
	       "\">" .
	       "<font size=2><img src=\"$imagesurl/up_arrow.gif\"" .
               "align=absmiddle border=0>" .
               "<b>Return to Thumbnails</b></font></a>";

    }
    else {
	$orig1 = $orig2 = $up1 = $up2 = "";
    }
    
    # print top
    print PAGE <<EOF;
<HTML>
  <HEAD><TITLE>$title - @{[$i+1]}/@{[@files+0]}</TITLE>
  </HEAD>
  <BODY bgcolor="#000000" text="#cccccc" link="#ffffff" vlink="#999999"
  alink="#ff0000">
<center>
  <table border=0>
EOF

    # print photo num and photo
    if (! $just_one){
	print PAGE <<EOF;
<tr>
  <td align=center>
    <font face="Verdana, Arial, Helvetica" size=2 color="#999999">
    <b>@{[ $i+1 ]}</b> of
    <b>@{[ @files+0 ]}</b> <font size=1>&nbsp; &nbsp;
    Click photo
    to continue<sub>&nbsp;</sub> $orig1 $up1
    </font></font><br><a href="@{[ $htmls[($i+1)%@files] ]}"><img
    src="$files[$i]"
    alt="<!--#include file=\'$descs[$i]\' -->" border=0></a></td>
</tr>
EOF
    }
    else {
	print PAGE <<EOF;
<tr>
  <td align=center>
    <img src="$files[$i]" alt=photo border=0></a></td>
</tr>
EOF
    }
    
    # print description
    print PAGE <<EOF;
<tr>
  <td align=center><table cellpadding=0 cellspacing=0 border=0 width=480>
    <tr>
      <td align=center>
        <font face="Verdana, Arial, Helvetica" size=2>
         <!--#include file='$descs[$i]' -->
	 <br>&nbsp;
        </font>
      </td>
    </tr></table></td>
</tr>
EOF
   
    # skip bottom links if just one image
    print "Just one: ";
    if (! $just_one){

	print "No";
	
	# print back link
	print PAGE <<EOF;
<tr>
  <td align=left><table cellpadding=0 cellspacing=0 border=0>
    <tr>
      <td width=125 align=center valign=top><a
        href="$htmls[($i-1)%@files]"><font face="Verdana, Arial, Helvetica"
        size=4><b><img src="$imagesurl/left_arrow.gif" align=absmiddle
	border=0>@{[
            ($i==0)?"<font size=\"-1\"><i>Last</i></font>":"Prev"
        ]}</b></font><br><img
        src="$thumbs[($i-1)%@files]" alt=next align=top></a></td>
      <td>&nbsp; &nbsp;</td>
EOF

	# print forward link
	print PAGE <<EOF;
      <td width=125 align=center valign=top><a
        href="$htmls[($i+1)%@files]"><font face="Verdana, Arial, Helvetica"
        size=4><b>@{[
            ($i==(@files-1))?"<font size=\"-1\"><i>First</i></font>":"Next"
        ]}<img src="$imagesurl/right_arrow.gif"
        align=absmiddle	border=0></b></font><br><img
        src="$thumbs[($i+1)%@files]" alt=next align=top></a></td>
      <td>&nbsp; &nbsp;</td>
EOF
      
	# print bottom
	print PAGE <<EOF;
      <td align=left valign=top><font
        face="Verdana, Arial, Helvetica" size=2>Viewing photo
        <b>@{[ $i+1 ]}</b> of
        <b>@{[ @files+0 ]}</b><br>
	<font size=1>Click photo to continue</font>
	<br>$orig2
	<br>$up2
	</font></td>
    </tr></table></td>
</tr>
EOF
    }
    else {
	print "Yes";
    }
    print ". ";
       

    # print bottom
    print PAGE <<EOF;
</table>
</center>
</body>
</html>
EOF

    # print out thumbnail links
    $_ = $ENV{PWD};
    s(^.*public_html/)(http://www.cs.fsu.edu/~beason/);
    print LINKS "<a href=\"$_/$htmls[$i]\"><img src=\"$_/",
        $thumbs[$i], "\"></a>\n";

    # close file
    close PAGE;
    
    # set permissions
    chmod(0711, ".");
    chmod(0644, "$files[$i]") or die $!;
    chmod(0644, "$htmls[$i]") or die $!;
    chmod(0644, "$thumbs[$i]") or die $!;
    chmod(0644, "$descs[$i]") or die $!;

    # say done
    print "Done.\n"
}

close LINKS;
chmod(0644, "links.html") or die $!;

# create page of thumbnails
if (! $just_one){
    print "Generating index of thumbnails. ";
    
    # Calculate numbering
    $_ = @htmls/$rows/$cols;
    my $index_total = int($_);
    if (($_-$index_total) >0.0){
	$index_total++;
    }
    my $index_next;
    my $index_prev;
    
    for ($i=1; @htmls; $i++){
	
	# open file
	open(INDEX, ">index$i.html") or die $!;

	# print header
	print INDEX <<EOF;
<HTML>
  <HEAD><TITLE>$title - Page $i</TITLE>
  </HEAD>
  <BODY bgcolor="#000000" text="#cccccc" link="#ffffff" vlink="#999999"
  alink="#ff0000">
<center>
EOF

	# print title
	print INDEX <<EOF;
<font face="Verdana, Arial, Helvetica"><font
size=4><b>$title</b></font><p><font size=2><b>Page:</b></font> &nbsp; 
EOF
	
	# print links
	$index_prev = $i==1 ? $index_total : $i-1;
	$index_next = $i==$index_total ? 1 : $i+1;

	print INDEX "<a href=\"index$index_prev.html\"><img src=\"" .
	    "$imagesurl/left_arrow.gif\" align=absmiddle border=0" .
	    "height=20 width=20 alt=\"Previous Page\"></a>";
	for ($k=1; $k<=$index_total; $k++){
	    if (0 || $k==$i){
		print INDEX " <font size=4 color=\"#666666\"><b>$k</b></font> ";
	    }
	    else {
		print INDEX " <a href=\"index$k.html\">" .
	            "<b><font size=2><span style=\"vertical-align: 15%;\">$k</span></font></b>" .
		    "</a> ";
	    }
	}
	print INDEX "<a href=\"index$index_next.html\"><img src=\"" .
	    "$imagesurl/right_arrow.gif\" align=absmiddle border=0 " .
	    "height=20 width=20 alt=\"Next Page\"></a></font>";
	
	
	# print table
	print INDEX <<EOF;
  <p><table border=0 cellspacing=20>
EOF

	# print rows
	for ($j=0; $j<$rows && @htmls; $j++){
	    print INDEX "<tr>\n";
	  
	    # print cols
	    for ($k=0; $k<$cols; $k++){
		print INDEX "<td valign=middle align=center width=102 height=102>";
	    
		if (@htmls){
		    # print thumbnail
		    print INDEX "<a href=\"$htmls[0]\"><img src=\"" .
			        "$thumbs[0]\" border=1 align=absmiddle " .
				"alt=\"" .
				"<!--#include file='$descs[0]' -->\">" .
     "<script>var desc=\"<!--#include file='$descs[0]' -->\";" .
     "if (desc.length)" .
     "  document.write('<font face=\"Verdana, Arial, Helvetica\" size=1 ' +" .
     "                 'style=\"vertical-align: top; text-decoration: none;" .
     "\"><br>comment</font>');" .
     "</script></a>";

		    shift @thumbs;
		    shift @htmls;
		    shift @descs;

		}

		print INDEX "</td>\n";
	    }
	    print INDEX "</tr>\n";
	}

	# print bottom
	print INDEX "</table>\n";
	
	# print footer
	print INDEX <<EOF;
</center>
</body>
</html>
EOF
    
	close INDEX;
        chmod(0644, "index$i.html") or die $!;
    }
    print "Done.\n";


# link index.html
system("/bin/rm", "index.html") if (-e "index.html" && -l "index.html");
system("/bin/ln", "-s", "index1.html", "index.html");
}

# syncronize to websrv
my $sync = "/home/grads/beason/bin/web_sync";
if (-x $sync){
    print "Syncing.\n";
    system($sync);
}
